--============================================================================--
-- memory.e e-mail: hayden_mckay@hotmail.com

--      - p=align(s,a)      - allocates an aligned space in physical memory
--      - here's how it works for those who have'nt a clue about alignments

--      - miss aligned dwords in memory
--      - |x..|...|...|...|...|...|...|...|...|xxx|xxx|xxx| -- cpu cache
--      -   |...,...,...,...,...,...,...|                   -- allocated
--
--      - aligned dwords in memory
--      - |x..|...|...|...|...|...|...|...|...|xxx|xxx|xxx| -- cpu cache
--      -   |.|...,...,...,...,...,...,...|.|               -- allocated
--
--      - NOTE: aligned memory can not be free'd with free() in machine.e
--              an atempt to free the pointer will cause a causeway error
--
--      - a=peekXu(p,s)     - 'p' is address  's' is size
--      - a=peekXs(p,s)     - 'p' is address  's' is size
--      - pokeXu(p,x,s)     - 'p' is address  'x' is data 's' is size
--      - pokeXs(p,x,s)     - 'p' is address  'x' is data 's' is size

--      peekx - pokex either 1,2,3 or 4 bytes
--      wrote it to replace peek/poke in if..elsif..end if statements

--      poke2() peek2u() peek2s()   - peek and poke words into memory
--      same rules as euphoria's poke4() peek4u() peek4s() procedures

include machine.e
include dll.e

sequence
machine_code
machine_code={
#55,#89,#E5,#52,#51,#53,#56,#57,#8B,#75,#08,#8B,#5D,#0C,#31,#C0,#83,#FB,#04,#74,
#0F,#83,#FB,#03,#74,#0E,#83,#FB,#02,#74,#2C,#83,#FB,#01,#74,#2C,#8B,#06,#EB,#2C,
#B8,#7F,#FF,#00,#00,#3C,#7F,#EB,#0D,#8A,#66,#02,#8A,#46,#01,#C1,#E0,#08,#8A,#06,
#EB,#16,#8A,#26,#8A,#46,#01,#C1,#E0,#08,#8A,#46,#02,#EB,#09,#66,#8B,#06,#EB,#04,
#8A,#06,#EB,#00,#5F,#5E,#5B,#59,#5A,#89,#EC,#5D,#C2,#00,#00,#90,#55,#89,#E5,#53,
#57,#8B,#7D,#08,#8B,#5D,#0C,#8B,#45,#10,#83,#F8,#04,#74,#0F,#83,#F8,#03,#74,#0E,
#83,#F8,#02,#74,#2C,#83,#F8,#01,#74,#2C,#89,#1F,#EB,#2C,#B8,#7F,#FF,#00,#00,#3C,
#7F,#74,#0D,#88,#1F,#88,#7F,#01,#C1,#EB,#08,#88,#7F,#02,#EB,#16,#88,#5F,#02,#88,
#7F,#01,#C1,#EB,#08,#88,#3F,#EB,#09,#66,#89,#1F,#EB,#04,#88,#1F,#EB,#00,#5F,#5B,
#59,#5A,#89,#EC,#5D,#C3,#90,#90,#55,#89,#E5,#56,#8B,#75,#08,#66,#8B,#06,#5E,#89,
#EC,#5D,#C2,#00,#00,#90,#90,#90,#55,#89,#E5,#57,#8B,#7D,#08,#8B,#45,#0C,#66,#89,
#07,#5F,#89,#EC,#5D,#C3}

global function align(integer s, integer a)
    atom addr

    addr=allocate(s+a)
    addr+=a-remainder(addr,a)

    return addr

end function

atom addr addr=align(226,4)
poke(addr,machine_code)

constant peekxu=define_c_func({},(addr+000),{C_POINTER,C_CHAR},C_UINT)
constant peekxs=define_c_func({},(addr+000),{C_POINTER,C_CHAR},C_LONG)
constant pokexu=define_c_proc({},(addr+096),{C_POINTER,C_UINT,C_CHAR})
constant pokexs=define_c_proc({},(addr+096),{C_POINTER,C_LONG,C_CHAR})
constant peekwu=define_c_func({},(addr+188),{C_POINTER},C_USHORT)
constant peekws=define_c_func({},(addr+188),{C_POINTER},C_SHORT )
constant pokewu=define_c_proc({},(addr+208),{C_POINTER,C_USHORT})

type sz(atom x) return x>=1 and x<=4 end type
type lp(object x) if atom(x) then return 1 else return length(x)=2 end if end type

global function  peekXu(atom p,sz s) return c_func(peekxu,{p,s})   end function
global function  peekXs(atom p,sz s) return c_func(peekxs,{p,s})   end function
global procedure pokeXu(atom p,atom x,sz s) c_proc(pokexu,{p,x,s}) end procedure
global procedure pokeXs(atom p,atom x,sz s) c_proc(pokexs,{p,x,s}) end procedure

global function peek2u(lp addr)
    object words
    
    if atom(addr) then words=c_func(peekwu,{addr})
    else
	words=repeat(0,addr[2])
	for i=1 to addr[2] do
	    words[i]=c_func(peekwu,{addr[1]})
	    addr[1]+=2
	end for
    end if
    
    return words
    
end function

global function peek2s(lp addr)
    object words

    if atom(addr) then words=c_func(peekws,{addr})
    else
	words=repeat(0,addr[2])
	for i=1 to addr[2] do
	    words[i]=c_func(peekws,{addr[1]})
	    addr[1]+=2
	end for
    end if

    return words

end function

global procedure poke2(atom addr, object data)

    if atom(data) then c_proc(pokewu,{addr,data})
    else
	for i=1 to length(data) do
	    c_proc(pokewu,{addr,data[i]})
	    addr+=2
      end for
    end if

end procedure
